@@ -128,7 +128,7 @@ static GnomeUIInfo menu_defaults[] = {
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_REVERT_TO_SAVED,
           0,  (GdkModifierType) 0, NULL },
 	/* Print */
-        { GNOME_APP_UI_ITEM, N_("_Print"), N_("Print the current file"),
+        { GNOME_APP_UI_ITEM, N_("_Print..."), N_("Print the current file"),
           NULL, NULL, NULL,
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_PRINT,
           GNOME_KEY_NAME_PRINT,  GNOME_KEY_MOD_PRINT, NULL },
@@ -152,7 +152,7 @@ static GnomeUIInfo menu_defaults[] = {
 	 * The "Edit" menu
 	 */
 	/* Cut */
-        { GNOME_APP_UI_ITEM, N_("C_ut"), N_("Cut the selection"),
+        { GNOME_APP_UI_ITEM, N_("Cu_t"), N_("Cut the selection"),
           NULL, NULL, NULL,
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_CUT,
           GNOME_KEY_NAME_CUT, GNOME_KEY_MOD_CUT, NULL },
@@ -188,13 +188,13 @@ static GnomeUIInfo menu_defaults[] = {
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_FIND,
           GNOME_KEY_NAME_FIND, GNOME_KEY_MOD_FIND, NULL },
 	/* Find Again */
-        { GNOME_APP_UI_ITEM, N_("Find _Again"),
+        { GNOME_APP_UI_ITEM, N_("Find Ne_xt"),
           N_("Search again for the same string"),
           NULL, NULL, NULL,
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_FIND,
           GNOME_KEY_NAME_FIND_AGAIN, GNOME_KEY_MOD_FIND_AGAIN, NULL },
 	/* Replace */
-        { GNOME_APP_UI_ITEM, N_("_Replace..."), N_("Replace a string"),
+        { GNOME_APP_UI_ITEM, N_("R_eplace..."), N_("Replace a string"),
           NULL, NULL, NULL,
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_FIND_AND_REPLACE,
           GNOME_KEY_NAME_REPLACE, GNOME_KEY_MOD_REPLACE, NULL },
@@ -208,7 +208,7 @@ static GnomeUIInfo menu_defaults[] = {
 	 * The Settings menu
 	 */
 	/* Settings */
-        { GNOME_APP_UI_ITEM, N_("_Preferences..."),
+        { GNOME_APP_UI_ITEM, N_("Prefere_nces..."),
           N_("Configure the application"),
           NULL, NULL,
 	  NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_PREFERENCES,
@@ -222,7 +222,7 @@ static GnomeUIInfo menu_defaults[] = {
           N_("About this application"), NULL, NULL,
 	  NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_ABOUT,
           0,  (GdkModifierType) 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Select All"),
+	{ GNOME_APP_UI_ITEM, N_("Select _All"),
           N_("Select everything"),
           NULL, NULL, NULL,
           GNOME_APP_PIXMAP_NONE, NULL,
@@ -329,6 +329,37 @@ static gchar *menu_names[] =
 };
 
 
+static GtkWidget *
+scale_pixbuf (GdkPixbuf *pixbuf, GtkIconSize icon_size) {
+	double pix_x, pix_y;
+	int width, height;
+
+	gtk_icon_size_lookup (icon_size, &width, &height);
+
+        pix_x = gdk_pixbuf_get_width (pixbuf);
+        pix_y = gdk_pixbuf_get_height (pixbuf);
+
+	/* Only scale if the image doesn't match the required
+	 * icon size 
+	 */
+        if (pix_x > width || pix_y > height) {
+        	double greatest;
+        	GdkPixbuf *scaled;
+		GtkWidget *image;
+
+                greatest = pix_x > pix_y ? pix_x : pix_y;
+                scaled = gdk_pixbuf_scale_simple (pixbuf,
+                                                  (width/ greatest) * pix_x,
+                                                  (height/ greatest) * pix_y,
+                                                   GDK_INTERP_BILINEAR);
+                image = gtk_image_new_from_pixbuf (scaled);
+                g_object_unref (G_OBJECT (scaled));
+		return image;
+	}
+	
+	return gtk_image_new_from_pixbuf (pixbuf);
+}
+
 /* Creates a pixmap appropriate for items.  */
 
 static GtkWidget *
@@ -351,7 +382,7 @@ create_pixmap (GnomeUIPixmapType pixmap_type, gconstpointer pixmap_info,
 			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_xpm_data
 				((const char **)pixmap_info);
 			if (pixbuf != NULL) {
-				pixmap = gtk_image_new_from_pixbuf (pixbuf);
+				pixmap = scale_pixbuf (pixbuf, icon_size);
 				g_object_unref (G_OBJECT (pixbuf));
 			}
 		}
@@ -370,7 +401,10 @@ create_pixmap (GnomeUIPixmapType pixmap_type, gconstpointer pixmap_info,
 			g_warning ("Could not find GNOME pixmap file %s",
 					(char *) pixmap_info);
 		else {
-			pixmap = gtk_image_new_from_file (name);
+			GdkPixbuf *pixbuf;
+			pixbuf = gdk_pixbuf_new_from_file (name, NULL);
+			pixmap = scale_pixbuf (pixbuf, icon_size);
+			g_object_unref (G_OBJECT (pixbuf));
 			g_free (name);
 		}
 
@@ -617,6 +651,7 @@ gnome_app_install_statusbar_menu_hints (GtkStatusbar* bar,
 		case GNOME_APP_UI_ITEM:
 		case GNOME_APP_UI_TOGGLEITEM:
 		case GNOME_APP_UI_SEPARATOR:
+		case GNOME_APP_UI_HELP:
 			install_menuitem_hint_to_statusbar(uiinfo, bar);
 			break;
 		case GNOME_APP_UI_RADIOITEMS:
@@ -818,6 +853,7 @@ gnome_app_install_appbar_menu_hints (GnomeAppBar* appbar,
 		case GNOME_APP_UI_ITEM:
 		case GNOME_APP_UI_TOGGLEITEM:
 		case GNOME_APP_UI_SEPARATOR:
+		case GNOME_APP_UI_HELP:
 			install_menuitem_hint_to_appbar(uiinfo, appbar);
 			break;
 		case GNOME_APP_UI_RADIOITEMS:
@@ -1089,20 +1125,36 @@ help_view_display_callback (GtkWidget *w, gpointer data)
 static int
 create_help_entries (GtkMenuShell *menu_shell, GnomeUIInfo *uiinfo, gint pos)
 {
-	GtkWidget *item;
-	GtkWidget *label;
+	gchar *path;
 
-	item = gtk_menu_item_new ();
-	label = create_label (_("_Help Contents"));
-	gtk_container_add (GTK_CONTAINER (item), label);
+	uiinfo->widget = gtk_image_menu_item_new_with_mnemonic (_("_Contents"));
+	uiinfo->hint = g_strdup (_("View help for this application"));
 
-	g_signal_connect_data (item, "activate",
+	setup_image_menu_item (uiinfo->widget, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_HELP);
+
+	g_signal_connect_data (uiinfo->widget, "activate",
 			       G_CALLBACK (help_view_display_callback),
 			       g_strdup (uiinfo->moreinfo), 
 			       (GClosureNotify) g_free, 0);
+	
+	gtk_menu_shell_insert (menu_shell, uiinfo->widget, pos);
 
-	gtk_menu_shell_insert (menu_shell, item, pos);
-	gtk_widget_show (item);
+	/* Install global accelerator */
+
+	path = g_strdup_printf("<%s>/help-contents-item", 
+			       gnome_program_get_app_id (gnome_program_get()));
+	
+	/* Associate the key combo with the accel path */
+	gtk_accel_map_add_entry (path, GDK_F1, 0);
+
+	/* Associate the accel path with the menu item */
+	if (GTK_IS_MENU (menu_shell))
+		gtk_menu_item_set_accel_path (GTK_MENU_ITEM (uiinfo->widget),
+					      path);
+
+	g_free (path);
+	
+	gtk_widget_show_all (uiinfo->widget);
 
 	return ++pos;
 }
@@ -2460,10 +2512,10 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 	toolbar_style = GTK_TOOLBAR_BOTH;
 	menu = gtk_menu_new ();
 
-	both = _("Both text and icons");
-	both_horiz = _("Both text and icons when horizontal, just icons when vertical");
-	icons = _("Icons only");
-	text = _("Text only");
+	both = _("Text Below Icons");
+	both_horiz = _("Text Beside Icons");
+	icons = _("Icons Only");
+	text = _("Text Only");
 
 	both_item = gtk_radio_menu_item_new_with_label (group, both);
 	g_signal_connect (both_item, "activate",
@@ -2472,7 +2524,7 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), both_item);
 
 	both_horiz_item = gtk_radio_menu_item_new_with_label (group, both_horiz);
-	g_signal_connect (both_item, "activate",
+	g_signal_connect (both_horiz_item, "activate",
 			  G_CALLBACK (style_menu_item_activated), GINT_TO_POINTER (GTK_TOOLBAR_BOTH_HORIZ));
 	group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (both_horiz_item));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), both_horiz_item);
